package data;

import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;

import javax.media.opengl.GL;



/**
 * @author "Daniel Pardo"
 * The node class represents nodes of the graph.<p>
 * Each node contains an id, a name, a size and a link to a graphic.
 */
public class Node extends Sprite{
	public static int overAllVisitedCounter = 0;
	public static int maxID = -1;
	private boolean isDeleted;
	public Node(int iD, float size, String name) {
		super(new Vector2(), new Vector2(size,size), "data"+File.separator+"pics"+File.separator+"default.png");
		ID = iD;
		Node.maxID = Math.max(maxID, ID);
		this.name = name;
		this.graphic = null;
		this.visitedCounter = overAllVisitedCounter;
		this.neighbors = new LinkedList<Node>();
		this.children = new LinkedList<Node>();
		this.others = new LinkedList<Node>();
		this.isDeleted = false;
	}
	
	public Node(int iD, float size, String name, String graphic) {
		super(new Vector2(), new Vector2(size,size), "data"+File.separator+"pics"+File.separator+graphic);
		ID = iD;
		Node.maxID = Math.max(maxID, ID);
		this.name = name;
		this.graphic = graphic;
		this.visitedCounter = overAllVisitedCounter;
		this.neighbors = new LinkedList<Node>();
		this.children = new LinkedList<Node>();
		this.others = new LinkedList<Node>();
		this.isDeleted = false;
	}
	@Override
	public boolean draw(GL gl){
		if(isDeleted)
			return false;
		return super.draw(gl);
	}
	/**
	 * returns the ID of the node
	 * @return ID
	 */
	public int getID() {
		return ID;
	}
	/**
	 * sets the ID of the node
	 * @param iD
	 */
	public void setID(int iD) {
		Node.maxID = Math.max(maxID, ID);
		ID = iD;
	}
	/**
	 * retuns the size of the node
	 * @return size
	 */
	public float getSize() {
		return size.x;
	}
	/**
	 * sets the size of the node
	 * @param size
	 */
	public void setSize(float size) {
		this.size.x = size;
		this.size.y = size;
	}
	/**
	 * return the name of the node
	 * @return name
	 */
	public String getName() {
		return name;
	}

	/**
	 * sets the name of the node
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * return the graphic of the node
	 * @return graphic
	 */
	public String getGraphic() {
		return graphic;
	}

	/**
	 * sets the graphic of the node
	 * @param graphic
	 */
	public void setGraphic(String graphic) {
		this.isInit = false;
		this.graphic = graphic;
		this.setImageFile("data"+File.separator+"pics"+File.separator+graphic);
	}
	
	private int ID;
	private String name;		
	private String graphic; // bsp: "human" fr link auf "human.svg" in unterordner /svg
	private int visitedCounter;

	private int ring;
	private float angle;
	private float sectorStart;
	private float sectorEnd;
	private int oldRing;
	private float oldAngle;
	
	private LinkedList<Node> neighbors;
	private LinkedList<Node> children;
	private LinkedList<Node> others;
	private Node father;
	
	private float angularWidth;
	
	public String toString()
	{
		String ret = "";
		ret = ret + "id: " + getID() + "  name: " + getName() + "  size: " + getSize() + "  graphic: " + ((getGraphic()==null)?"null":getGraphic());
		
		ret += "\nNeighbors: ";
		Iterator<Node>it = neighbors.iterator();
		while(it.hasNext())
			ret += it.next().getID()+", ";
		
		if(father!=null)
			ret += "\nFather: "+father.getID();
		
		ret += "\nChildren: ";
		it = children.iterator();
		while(it.hasNext())
			ret += it.next().getID()+", ";
		
		ret += "\nOthers: ";
		it = others.iterator();
		while(it.hasNext())
			ret += it.next().getID()+", ";
		return ret;
	}
	/**
	 * return the visitedCounter(used by GraphProcessing) of the node
	 * @return visitedCounter
	 */
	public int getVisitedCounter() {
		return visitedCounter;
	}

	/**
	 * sets the visitedCounter(used by GraphProcessing) of the node
	 * @param visitedCounter
	 */
	public void setVisitedCounter(int visitedCounter) {
		this.visitedCounter = visitedCounter;
	}

	/**
	 * return the ring on that the node lays
	 * @return ring
	 */
	public int getRing() {
		return ring;
	}

	/**
	 * sets the ring on that the node lays
	 * @param ring
	 */
	public void setRing(int ring) {
		this.ring = ring;
	}

	/**
	 * return the angle of the node
	 * @return angle
	 */
	public float getAngle() {
		return angle;
	}

	/**
	 * sets the angle of the node
	 * @param angle
	 */
	public void setAngle(float angle) {
		this.angle = angle;
	}

	/**
	 * return the sector start of the node
	 * @return sectorStart
	 */
	public float getSectorStart() {
		return sectorStart;
	}

	/**
	 * sets the sector start of the node
	 * @param sectorStart
	 */
	public void setSectorStart(float sectorStart) {
		this.sectorStart = sectorStart;
	}

	/**
	 * return the sector end of the node
	 * @return sectorEnd
	 */
	public float getSectorEnd() {
		return sectorEnd;
	}

	/**
	 * sets the sector end of the node
	 * @param sectorEnd
	 */
	public void setSectorEnd(float sectorEnd) {
		this.sectorEnd = sectorEnd;
	}

	/**
	 * return the old ring of the node
	 * @return oldRing
	 */
	public int getOldRing() {
		return oldRing;
	}

	/**
	 * sets the old ring of the node
	 * @param oldRing
	 */
	public void setOldRing(int oldRing) {
		this.oldRing = oldRing;
	}

	/**
	 * return the old angle of the node
	 * @return oldAngle
	 */
	public float getOldAngle() {
		return oldAngle;
	}

	/**
	 * sets the old angle of the node
	 * @param oldAngle
	 */
	public void setOldAngle(float oldAngle) {
		this.oldAngle = oldAngle;
	}

	@Override
	public boolean init(GL gl) {
		return super.init(gl);
	}

	/**
	 * sets the neighbors of the node
	 * @param neighbors
	 */
	public void setNeighbors(LinkedList<Node> neighbors) {
		this.neighbors = neighbors;
	}

	/**
	 * return the neighbors of the node
	 * @return neighbors
	 */
	public LinkedList<Node> getNeighbors() {
		return neighbors;
	}

	/**
	 * return the children of the node
	 * @return children
	 */
	public LinkedList<Node> getChildren() {
		return children;
	}

	/**
	 * sets the children of the node
	 * @param children
	 */
	public void setChildren(LinkedList<Node> children) {
		this.children = children;
	}

	/**
	 * return the others of the node
	 * @return others
	 */
	public LinkedList<Node> getOthers() {
		return others;
	}

	/**
	 * sets the others of the node
	 * @param others
	 */
	public void setOthers(LinkedList<Node> others) {
		this.others = others;
	}

	/**
	 * return the father of the node
	 * @return father
	 */
	public Node getFather() {
		return father;
	}

	/**
	 * sets the father of the node
	 * @param father
	 */
	public void setFather(Node father) {
		this.father = father;
	}

	/**
	 * return if the node is/shall be deleted
	 * @return is deleted?
	 */
	public boolean isDeleted() {
		return isDeleted;
	}

	/**
	 * sets if the node is/shall be deleted
	 * @param isDeleted
	 */
	public void setDeleted(boolean isDeleted) {
		this.isDeleted = isDeleted;
	}

	/**
	 * sets the angular width of the node
	 * @param angularWidth
	 */
	public void setAngularWidth(float angularWidth) {
		this.angularWidth = angularWidth;
	}

	/**
	 * return the angular width of the node
	 * @return angularWidth
	 */
	public float getAngularWidth() {
		return angularWidth;
	}
	
}
