package logic;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;

import com.sun.opengl.util.j2d.TextRenderer;

import ui.GUI;
import ui.GUI.NodeAction;

import data.Camera;
import data.Drawable;
import data.Node;
import data.Sprite;
import data.Vector2;

/**
 * Handler for GLCanvas, OpenGL and user input
 * @author Felix Flei (MatrNr: 0527038)
 *
 */
public class CanvasHandler implements GLEventListener, MouseMotionListener,
MouseListener, KeyListener, MouseWheelListener{
	private int[] oldMousePos = {-1, -1};
	private Camera camera;
	public LinkedList<Drawable>drawList;
	public enum SurfaceDrawType{FLAT, SMOOTH};
	public boolean useLight;
	public SurfaceDrawType surfaceDrawType;
	private long oldTime;
	private Vector2 clickPosition;
	private GL gl;
	private int screenScaler;
	private int width;
	private int height;
	private TextRenderer textRenderer;
	private static boolean showText = true;
	/**
	 * initializes CanvasHandler
	 */
	public CanvasHandler(){
		this(new LinkedList<Drawable> ());
	}
	/**
	 * initializes CanvasHandler by assigning a draw-able list
	 * @param drawList list of objects to draw (calls draw-function)
	 */
	public CanvasHandler(LinkedList<Drawable> drawList){
		this.drawList = drawList;
		clickPosition = null;
	}
	/**
	 * adds a drawable to the end of the list
	 * @param drawable
	 */
	public void addDrawable(Drawable drawable){
		this.drawList.add(drawable);
	}
	/**
	 * adds a drawable to the beginning of the list
	 * @param drawable
	 */
	public void addDrawableBeforeFirst(Drawable drawable){
		this.drawList.add(0, drawable);
	}
	/**
	 * removes a specific drawable
	 * @param drawable
	 */
	public void removeDrawable(Drawable drawable){
		this.drawList.remove(drawable);
	}
	/**
	 * replaces the whole drawable list of the CanvasHandler with the given one
	 * @param drawList
	 */
	public void setDrawableList(LinkedList<Drawable> drawList){
		this.drawList = drawList;
		Iterator<Drawable>iter = drawList.iterator();
		while(iter.hasNext())
			iter.next().init(gl);
	}
	/**
	 * configures OpenGL
	 */
	public void init(GLAutoDrawable arg0)     {
		gl = arg0.getGL();

		gl.glEnable(GL.GL_DOUBLEBUFFER);
		gl.glEnable(GL.GL_LIGHTING);
		gl.glEnable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_COLOR_MATERIAL);
		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glEnable(GL.GL_NORMALIZE);
		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glEnable(GL.GL_BLEND);
		gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
		//gl.glPolygonMode( GL.GL_FRONT_AND_BACK, GL.GL_LINE );

		gl.glClearColor(0.7f, 0.7f, 0.7f, 0.0f);

		gl.glMatrixMode(GL.GL_PROJECTION);
		//gl.glOrtho(-100, 100, -100, 100, -100, 100);

		gl.glMatrixMode(GL.GL_MODELVIEW);
		//gl.glRotatef(35.0f, 1.0f, 0.0f, 0.0f);    // Rotation um die x-Achse
		//gl.glRotatef(-25.0f, 0.0f, 1.0f, 0.0f);   // Rotation um die y-Achse
		double[] pos = {0.0, 0.0, 0.0};
		double pan = 0.0;
		double tilt = 0.0;
		double speed = 0.001;
		double zoom = 0.1;
		this.camera = new Camera(pos, pan, tilt, zoom, speed, gl);
		this.surfaceDrawType = SurfaceDrawType.SMOOTH;
		this.oldTime = Long.MIN_VALUE;
		this.useLight = true;
		Iterator<Drawable>iterator = this.drawList.iterator();
		while(iterator.hasNext()){
			boolean ret = iterator.next().init(gl);
			if (ret == false)
				iterator.remove();
		}

		String []imagesStrings = (new File("data"+File.separator+"pics")).list();
		for(int i = 0; i < imagesStrings.length; i++){
			Sprite.loadTexture("data"+File.separator+"pics"+File.separator+imagesStrings[i]);
		}
		textRenderer = new TextRenderer( new Font( "SansSerif", Font.PLAIN, 15 ),
               false, false, null );
		//for(int i =0; i<100; i++)
		//	this.drawList.add(new Sprite(new Vector2((float)Math.random()*2-1, (float)Math.random()*2-1), new Vector2((float)Math.random(), (float)Math.random()), "test.png"));
	}
	/**
	 * internal used function for GLCanvas to display (render) drawables
	 */
	@Override
	public void display(GLAutoDrawable arg0)     {
		GL gl = arg0.getGL();
		long currentTime = System.currentTimeMillis();
		if(this.oldTime == Long.MAX_VALUE) this.oldTime = currentTime;
		long deltaTime = currentTime - oldTime;
		oldTime = currentTime;

		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		gl.glPushMatrix();
		gl.glLoadIdentity();
		camera.update(deltaTime);
		camera.transformIntoCameraSpace();
		Node node;
		Vector2 textPos = new Vector2();
		for(java.util.Iterator<Drawable> it = this.drawList.iterator(); it.hasNext();){
			Drawable d  = it.next();
			gl.glColor3d(1.0, 1.0, 1.0);
			boolean ret = d.draw(gl);
			if (ret == false)
				it.remove();
			if(showText){
				if(d instanceof Node){
					textRenderer.beginRendering( width, height );
					textRenderer.setColor( 0.0f, 0.0f, 0.0f, 0.7f );
					node = (Node)d;
					textPos.set(node.getPosition().x, node.getPosition().y);
					textPos.y = textPos.y;
					textPos.y += camera.getPos()[1];
					textPos.x += camera.getPos()[0];
					textPos.mult((float)camera.getZoom());
					textPos.mult(screenScaler/2.0f);
					textPos.x += this.width/2;
					textPos.y += this.height/2;
					textRenderer.draw( node.getName(), (int)textPos.x, (int)textPos.y );
					textRenderer.endRendering();
				}
			}
		}
		gl.glPopMatrix();
	}
	@Override
	public void reshape(GLAutoDrawable arg0, int arg1, int arg2, int width, int height) {
		GL gl = arg0.getGL();
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		this.width  = width;
		this.height  = height;
		//System.out.println("width "+ width+"; height"+height);
		if(height>width){
			gl.glOrtho(-1, 1, -(double)height/(double)width, (double)height/(double)width, -10.0, 10.0);
			screenScaler = width;
		}else{
			gl.glOrtho( -(double)width/(double)height, (double)width/(double)height, -1, 1,-10.0, 10.0);
			screenScaler = height;
		}
		gl.glMatrixMode(GL.GL_MODELVIEW);
	}
	@Override
	public void displayChanged(GLAutoDrawable arg0, boolean arg1, boolean arg2)     {
	}
	@Override
	public void mouseDragged(MouseEvent e) {
		if(oldMousePos[0] != -1 && oldMousePos[1] != -1){
			float x = (e.getX()-oldMousePos[0])/450.0f/(float)camera.getZoom();
			float y = (e.getY()-oldMousePos[1])/450.0f/(float)camera.getZoom();
			this.oldMousePos[0] = e.getX();
			this.oldMousePos[1] = e.getY();
			this.camera.move(x, -y, 0);
		}
	}
	@Override
	public void mouseMoved(MouseEvent e) {
	}
	@Override
	public void mouseClicked(MouseEvent e) {
	}
	@Override
	public void mouseEntered(MouseEvent e) {
	}
	@Override
	public void mouseExited(MouseEvent e) {
	}
	@Override
	public void mousePressed(MouseEvent e) {
		if(e.getButton() == MouseEvent.BUTTON1){
			this.oldMousePos[0] = e.getX();
			this.oldMousePos[1] = e.getY();
		}
	}
	@Override
	public void mouseReleased(MouseEvent e) {
		if(e.getButton() == MouseEvent.BUTTON1){
			if(e.getClickCount() > 1){
				clickPosition = new Vector2(e.getX(), e.getY());
				clickPosition.x -= this.width/2;
				clickPosition.y -= this.height/2;
				clickPosition.mult(2.0f/screenScaler);
				clickPosition.mult(1.0f/(float)camera.getZoom());
				clickPosition.x -= camera.getPos()[0];
				clickPosition.y += camera.getPos()[1];
				clickPosition.y = -clickPosition.y;
				GUI.selectNodeAndPerformNodeAction(clickPosition);
			}else{
				clickPosition = null;
			}
			this.oldMousePos[0] = -1;
			this.oldMousePos[1] = -1;
		}
	}
	@Override
	public void keyPressed(KeyEvent e) {
		this.moveCamera(e.getKeyCode(), true);
	}
	@Override
	public void keyReleased(KeyEvent e) {
		this.moveCamera(e.getKeyCode(), false);
	}
	@Override
	public void keyTyped(KeyEvent e) {
	}
	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		this.camera.addZoom(-e.getUnitsToScroll()/10.0f*camera.getZoom());
	}
	/**
	 * moves the camera (W/A/S/D/Q/E)
	 * @param keyCode
	 * @param move if true, camera moves, if false, camera stops into the direction specified by keyCode
	 */
	private void moveCamera(int keyCode, boolean move){
		switch (keyCode){
		case KeyEvent.VK_ESCAPE:
			GUI.setNodeAction(NodeAction.NONE);
		}
	}
	/**
	 * returns the jogl GL context
	 * @return gl
	 */
	public GL getGL(){
		return this.gl;
	}
	/**
	 * returns if true if text shall be displayed, else returns false
	 * @return is text shown?
	 */
	public static boolean isShowText() {
		return showText;
	}
	/**
	 * defines if text is displayed in the canvas
	 * @param showText if true, the text is shown, otherwise not
	 */
	public static void setShowText(boolean showText) {
		CanvasHandler.showText = showText;
	}
	
}
