package logic;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Stack;

import data.Node;
import data.Rings;
import data.Vector2;


public class GraphProcessing {
	private Node focusNode;
	/**
	 * main part of the algorithm.
	 * generates a tree out of the graph 
	 * and sets the angle and ring at which the node has to be placed.
	 * @param focusNode
	 */
	private boolean graphGeneratedOnce = false;
	Node fatherNode;
	public void generateTree(Node focusNode){
		this.focusNode = focusNode;
		
		fatherNode = focusNode.getFather();
		this.focusNode.setFather(null);
		int visitedCounter = Node.overAllVisitedCounter;
		focusNode.setVisitedCounter(visitedCounter+1);
		Stack<Node> parents;
		Stack<Node> children = new Stack<Node>();
		children.push(focusNode);
		LinkedList<Node>neighbors;
		Iterator<Node>iterator;
		Node currentNode;
		Node neighbor;
		int ring = 0;
		int generationSize = 0;
		while(!children.isEmpty())
		{
			parents = children;
			children = new Stack<Node>();
			generationSize = parents.size();
			while(!parents.isEmpty())
			{
				currentNode = parents.pop();
				currentNode.setOldAngle(currentNode.getAngle());
				currentNode.setOldRing(currentNode.getRing());
				currentNode.setRing(ring);
				//currentNode.setSector((float)(2*Math.PI / generationSize));
				neighbors = currentNode.getNeighbors();
				iterator = neighbors.iterator();
				LinkedList<Node> currentChildren = new LinkedList<Node>();
				LinkedList<Node> currentOthers = new LinkedList<Node>();
				while(iterator.hasNext())
				{
					neighbor = iterator.next();
					if(neighbor.getVisitedCounter() == visitedCounter){//neighbor is child
						children.push(neighbor);
						currentChildren.add(neighbor);
						neighbor.setFather(currentNode);
						neighbor.setVisitedCounter(visitedCounter+1);
					}else if (currentNode.getFather() != neighbor){//neighbor is not child and not father
						currentOthers.add(neighbor);
					}
				}
				currentNode.setChildren(currentChildren);
				currentNode.setOthers(currentOthers);
				//System.out.println(currentNode);
			}
			ring++;
		}
		Rings.setRingNumber(ring);
		Node.overAllVisitedCounter = visitedCounter+1;
		
		if (graphGeneratedOnce)
		{
			if (fatherNode == null)
				graphGeneratedOnce = false;
		}
		
		
		
		DepthFirstIterator nodeItDF = Visualization.getGraphProcessing().iteratorDepthFirst();
		//compute angular width of each node - start with leave nodes, traverse other nodes backwards using a stack
		parents = new Stack<Node>();	
		while (nodeItDF.hasNext())
		{
			Node node = nodeItDF.next();
			if (!node.getChildren().isEmpty())
			{
				parents.push(node); //save parents on stack to traverse them backwards later...
			}
			else //node has no children
			{
				float angularWidth = (float) (node.getSize()/Visualization.getRadiusOfRing(node.getRing()));
				node.setAngularWidth(angularWidth);
			}
		}
		while (parents.size() > 0) //now traverse parents backwards
		{
			Node parent = parents.pop();
			Iterator<Node> it = parent.getChildren().iterator();
			
			float sumAngularWidthOfSubtree = 0;
			while (it.hasNext()) // get sum of the childs angular widths
			{
				Node child = it.next();
				sumAngularWidthOfSubtree += child.getAngularWidth();
			}
			float angularWidthParentNode = 0;
			if (parent.getRing() != 0)
			{
				angularWidthParentNode = (float) (parent.getSize()/Visualization.getRadiusOfRing(parent.getRing()));
			}
			else // parent node is focus node is root node
			{
				angularWidthParentNode = 0;
			}
				
			//max of parents and subtrees angular width
			parent.setAngularWidth( Math.max(angularWidthParentNode, sumAngularWidthOfSubtree) ); 
		}
		
		//iterate tree - normalizing angular widths of each node
		float normalizeFactor = (float) (2*Math.PI / Visualization.getFocusNode().getAngularWidth());	
		nodeItDF = Visualization.getGraphProcessing().iteratorDepthFirst();
		while (nodeItDF.hasNext())
		{
			Node node = nodeItDF.next();
			node.setAngularWidth(node.getAngularWidth() * normalizeFactor);
		}
		
		//updating angular width if children too small
		nodeItDF = Visualization.getGraphProcessing().iteratorDepthFirst();
		while (nodeItDF.hasNext())
		{
			Node node = nodeItDF.next();
			if (!node.getChildren().isEmpty())
			{
				float sum = 0;
				Iterator<Node> it = node.getChildren().iterator();
				while (it.hasNext())
				{
					Node child = it.next();
					sum += child.getAngularWidth();
				}
				if (sum < node.getAngularWidth())
				{
					it = node.getChildren().iterator();
					while (it.hasNext())
					{
						Node child = it.next();
						child.setAngularWidth(child.getAngularWidth()/sum*node.getAngularWidth());
					}
					
				}
			}
		}
		
		
		//set angles of nodes
		BreadthFirstIterator nodeItBF = Visualization.getGraphProcessing().iteratorBreadthFirst();
		
		int gen = 0;
		LinkedList<Node> parents1 = new LinkedList<Node>();
		LinkedList<Node> currentNodeList = new LinkedList<Node>();
		if (nodeItBF.hasNextGeneration())
		{
			currentNodeList = nodeItBF.nextGeneration();
			if (nodeItBF.hasNext())
			{
				Node node = nodeItBF.next();			
				
				if (fatherNode!=null)
				{
					Vector2 fokusVec = focusNode.getPosition();
//					fokusVec = new Vector2(focusNode.getOldRing()*(float)Math.cos(focusNode.getOldAngle()), 
//							focusNode.getOldRing()*(float)Math.sin(focusNode.getOldAngle()) );
					
					Vector2 fatherVec = new Vector2(0.0f, 0.0f);
					fatherVec = fatherNode.getPosition();
					
					
//					fatherVec = new Vector2(fatherNode.getOldRing()*(float)Math.cos(fatherNode.getOldAngle()), 
//							fatherNode.getOldRing()*(float)Math.sin(fatherNode.getOldAngle()) );
					
					Vector2 res = new Vector2( fatherVec.x - fokusVec.x, fatherVec.y - fokusVec.y);
					//Vector2 res = fokusVec.clone().sub(fatherVec);
					
					float diffAngle = 0;
					if (res.x > 0)
					{
						if (res.y >= 0)
							diffAngle = (float) Math.atan(res.y/res.x);
						else
							diffAngle = (float) Math.atan(res.y/res.x) + 2.0f * (float) Math.PI;
					} else if (res.x < 0) {
						diffAngle = (float) Math.atan(res.y/res.x) + (float) Math.PI;
					} else {
						if (res.y > 0)
							diffAngle = (float) Math.PI / 2.0f;
						else {
							if (res.y < 0)
								diffAngle = (float) 3.0f * (float) Math.PI / 2.0f;
							else
								diffAngle = 0;
						}
					}
		
					//node.setSectorStart(0 + diffAngle + (float) Math.PI/2 - fatherNode.getAngle());
					//node.setSectorEnd((float) (2*Math.PI) + diffAngle + (float) Math.PI/2 - fatherNode.getAngle());
					
					//node.setSectorStart(0 + diffAngle - fatherNode.getAngle());
					//node.setSectorEnd((float) (2*Math.PI) + diffAngle - fatherNode.getAngle());
					
					float deg = (float) (diffAngle*360.0f/(2*Math.PI));
					
					//transform to other coord system (360 - (angle - 90))
					float val = (2*(float)Math.PI - (diffAngle-(float)Math.PI/2) );
					
					node.setSectorStart( 0 + val - fatherNode.getAngularWidth()/2 );
					node.setSectorEnd( (float) (2*Math.PI) + val - fatherNode.getAngularWidth()/2 );
				}
				else
				{
				node.setSectorStart(0);
				node.setSectorEnd((float) (2*Math.PI));
				}
				
//				node.setSectorStart(0);
//				node.setSectorEnd((float) (2*Math.PI));
				
				//node.setAngle(0);
				//node.setOldAngle(Visualization.getFocusNode().getAngle());
			}
			gen++;
			
		}
		while (nodeItBF.hasNextGeneration())
		{
			parents1 = currentNodeList;
			currentNodeList = nodeItBF.nextGeneration();
			Iterator<Node> it = currentNodeList.iterator();
			
			LinkedList<Float> indList = new LinkedList<Float>();
			for (int i=0; i<parents1.size(); i++)
				indList.push(0.0f);
			
			if (gen == 1)
			{
				if (graphGeneratedOnce)
				{
					//float val = indList.pop() + fatherNode.getAngle();
					//indList.push( val );
				}
			}
			
			while (it.hasNext())
			{
				Node node = it.next();
	
				int index = parents1.indexOf(node.getFather());
				float factorStart = (indList.get(index));
				
				indList.set(index, (indList.get(index)) +  node.getAngularWidth()); //increase index
				
				node.setSectorStart(
						factorStart + node.getFather().getSectorStart() );
				node.setSectorEnd(
						factorStart + node.getFather().getSectorStart() + node.getAngularWidth() );
				node.setAngle(
						factorStart + node.getFather().getSectorStart() + node.getAngularWidth()/2 );

//				node.setOldAngle(
//						factorStart + node.getFather().getSectorStart() + node.getAngularWidth()/2 );
			
			}
			gen++;
		}
		
		
		
		graphGeneratedOnce = true;
		
	}
	/**
	 * returns an iterator, that traverses the graph like a tree with breadth-first search.
	 * so gernerateTree() has to be called previously
	 * @return iterator
	 */
	public BreadthFirstIterator iteratorBreadthFirst(){
		return new BreadthFirstIterator();
	}
	/**
	 * a Breath First Iterator traversing the graph (gernerateTree() has to be called previously)
	 * @author Fennes
	 *
	 */
	public class BreadthFirstIterator implements Iterator<Node>{
		LinkedList<Node> parents;
		LinkedList<Node> children;
		Iterator<Node>iterator;
		public BreadthFirstIterator(){
			parents = new LinkedList<Node>();
			children = new LinkedList<Node>();
			iterator = parents.iterator();
		}
		/**
		 * returns true if there are remaining nodes in the current generation,
		 * else returns false
		 */
		@Override
		public boolean hasNext() {
			return iterator.hasNext();
		}
		/**
		 * returns the next node of the current generation
		 */
		@Override
		public Node next() {
			return iterator.next();
		}
		@Override
		public void remove() {
		}
		/**
		 * returns true if the current generation has any children and prepares nextGeneration()
		 * @return true if has next generation
		 */
		public boolean hasNextGeneration(){
			if(focusNode == null)
				return false;
			if(parents.isEmpty()){
				children.add(GraphProcessing.this.focusNode);
				return true;
			}else{
				Iterator<Node>it = parents.iterator();
				children = new LinkedList<Node>();
				while(it.hasNext())
					children.addAll(it.next().getChildren());
				return !children.isEmpty();
			}
		}
		/**
		 * returns the next generation and sets this to the current generation.
		 * allows to proceed with hasNext() and next()
		 * @return Returns the next generation of Nodes in a LinkedList
		 */
		public LinkedList<Node> nextGeneration(){
			parents = children;
			iterator = parents.iterator();
			return parents;
		}
	}
	/**
	 * returns an iterator, that traverses the graph like a tree with depth-first search.
	 * so gernerateTree() has to be called previously
	 * @return iterator
	 */
	public DepthFirstIterator iteratorDepthFirst(){
		return new DepthFirstIterator();
	}
	/**
	 * a Depth First Iterator traversing the graph (gernerateTree() has to be called previously)
	 * @author Fennes
	 *
	 */
	public class DepthFirstIterator implements Iterator<Node>{
		HashMap<Node, Integer>childIndices;
		Node currentNode = null;
		public DepthFirstIterator(){
			childIndices = new HashMap<Node, Integer>();
		}
		@Override
		public boolean hasNext() {
			if(currentNode == null){
				//System.out.println("currentNode is null");
				currentNode = GraphProcessing.this.focusNode;
				childIndices.put(currentNode, 0);
				if(currentNode == null)
					return false;
				else
					return true;
			}else{
				//System.out.println("current node is: "+currentNode.toString());
				//System.out.println("child index is: "+childIndices.get(currentNode));
				while(childIndices.get(currentNode)+1 > currentNode.getChildren().size()){
					//System.out.println("index higher than size: "+currentNode.getChildren().size());
					currentNode = currentNode.getFather();
					if(currentNode == null)
						return false;
				}
				currentNode = currentNode.getChildren().get(childIndices.get(currentNode));
				childIndices.put(currentNode.getFather(), childIndices.get(currentNode.getFather())+1);
				//System.out.println("children index of father is: "+childIndices.get(currentNode.getFather()));
				childIndices.put(currentNode, 0);
				//System.out.println("children index is: "+childIndices.get(currentNode));
				if(currentNode != null)
					return true;
				else
					return false;
			}
		}
		@Override
		public Node next() {
			return currentNode;
		}
		@Override
		public void remove() {
		}
	}
	
	public Node getFocusNode() {
		return focusNode;
	}
	
	public void setFocusNode(Node focusNode) {
		this.focusNode = focusNode;
	}
	public void addChild(Node node, Node father){
		father.getNeighbors().add(node);
		father.getChildren().add(node);
		node.getNeighbors().add(father);
		node.setFather(father);
	}
	public void removeNode(Node node){
		Iterator<Node>iterator = node.getNeighbors().iterator();
		while(iterator.hasNext()){
			Node n = iterator.next();
			n.getNeighbors().remove(node);
		}
		node.setDeleted(true);
	}
	public void removeSubtree(Node node){
		Iterator<Node>iterator = node.getChildren().iterator();
		while(iterator.hasNext()){
			removeSubtree(iterator.next());
		}
		node.setDeleted(true);
	}
	public void addEdge(Node n1, Node n2){
		if(n1.getNeighbors().contains(n2) || n2.getNeighbors().contains(n1))
			return;
		n1.getNeighbors().add(n2);
		n1.getOthers().add(n2);
		n2.getNeighbors().add(n1);
		n2.getOthers().add(n1);
	}
}
